/* The Linux Channel
 * Code for Video Episode: 0x166 NVIDIA CUDA Toolkit - Parallel Programming in CUDA
 * Author: Kiran Kankipati
 * Updated: 04-Sep-2018
 *
 * Code based on and derived from: 
	https://www.olcf.ornl.gov/wp-content/uploads/2013/02/Intro_to_CUDA_C-TS.pdf
	References:
	https://developer.download.nvidia.com/books/cuda-by-example/cuda-by-example-sample.pdf
	https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html
	https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#programming-model
	http://www.hds.bme.hu/~fhegedus/C++/Professional%20CUDA%20C%20Programming.pdf
 *
 */



#include <stdio.h>


__global__ void add(int *a, int *b, int *c) { *c = *a + *b; }


int main(void) 
{	int a, b, c; //host copies of a, b, c
	int *d_a, *d_b, *d_c;//device copies of a, b, c
	int size = sizeof(int);
	cudaError_t err;
	
	err = cudaGetLastError(); 
	if(err != cudaSuccess) { printf("Error: %02x\n", (int)err); }
	
	// Allocate space for device copies of a, b, c
	cudaMalloc((void**)&d_a, size);
	cudaMalloc((void**)&d_b, size);
	cudaMalloc((void**)&d_c, size);
	
	err = cudaGetLastError();
	if(err != cudaSuccess) { printf("Error: %s\n", cudaGetErrorString(err)); }
	
	// Setup input values
	a = 2; b = 7; c=0;
	printf("BEFORE: %d, %d, %d\n", a, b, c);
	
	// Copy inputs to device
	cudaMemcpy(d_a, &a, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_b, &b, size, cudaMemcpyHostToDevice);
	
	// Launch add() kernel on GPU
	add<<<1, 1>>>(d_a, d_b, d_c);
	
	err = cudaGetLastError();
	if(err != cudaSuccess) { printf("Error: %s\n", cudaGetErrorString(err)); }
	
	// Copy result back to host
	cudaMemcpy(&c, d_c, size,cudaMemcpyDeviceToHost);
	
	printf("AFTER: %d, %d, %d\n", a, b, c);
	
	// Cleanup
	cudaFree(d_a); 
	cudaFree(d_b); 
	cudaFree(d_c);
	return 0;

}