/* The Linux Channel
 * Code for Video Episode: 0x166 NVIDIA CUDA Toolkit - Parallel Programming in CUDA
 * Author: Kiran Kankipati
 * Updated: 04-Sep-2018
 *
 * Code based on and derived from: 
	https://www.olcf.ornl.gov/wp-content/uploads/2013/02/Intro_to_CUDA_C-TS.pdf
	References:
	https://developer.download.nvidia.com/books/cuda-by-example/cuda-by-example-sample.pdf
	https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html
	https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#programming-model
	http://www.hds.bme.hu/~fhegedus/C++/Professional%20CUDA%20C%20Programming.pdf
 *
 */

#include <stdio.h>


__global__ void add(int *a, int *b, int *c) { c[blockIdx.x] = a[blockIdx.x] + b[blockIdx.x]; }

void random_ints(int* a, int N, int rand_max)
{  int i;
   for(i = 0; i < N; ++i) { a[i] = rand()%rand_max; }
}

void print_arr(const char *str, int* a) { printf("%s [0]=%u [1]=%u [2]=%u \n", str, a[0], a[1], a[2] ); }


#define N 512
int main(void)
{
	int *a, *b, *c; // host copies of a, b, c
	int *d_a, *d_b, *d_c; // device copies of a, b, c
	int size = N * sizeof(int);
	//Alloc space for device copies of a, b, c
	cudaMalloc((void**)&d_a, size);
	cudaMalloc((void**)&d_b, size);
	cudaMalloc((void**)&d_c, size);
	
	// Alloc space for host copies of a, b, c and setup input values
	a = (int *)malloc(size); random_ints(a, N, 10);
	b = (int *)malloc(size); random_ints(b, N, 10);
	c = (int *)malloc(size);
	
	print_arr("a", a); print_arr("b", b); 
	
	// Copy inputs to device
	cudaMemcpy(d_a, a, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_b, b, size, cudaMemcpyHostToDevice);
	
	// Launch add() kernel on GPU with N blocks
	add<<<N,1>>>(d_a, d_b, d_c);
	
	// Copy result back to host
	cudaMemcpy(c, d_c, size, cudaMemcpyDeviceToHost);
	
	print_arr("c", c); 
	
	// Cleanup
	free(a); free(b); free(c);
	cudaFree(d_a); cudaFree(d_b); cudaFree(d_c);
	
	return 0;
}


